/*
 * NAME:
 *	net_fgetln - interface stdio to read_net and write_net
 *
 * SYNOPSIS:
 *	#include <netio.h>
 *
 *	char *	net_fgetln(FILE *fp, size_t *n);
 *
 * DESCRIPTION:
 *	These routines simply allow encryption hooks to be used with
 *	applications that use stdio.
 *
 *	'netio.h' contains macro definitions which override fprintf(),
 *	vfprintf(), fgets(), fgetc(), getc(), putc() and fputc().
 *
 *	Each of the net_*() functions, checks whether the FILE * is
 *	associated with an SSL connection and if not calls the normal
 *	stdio function, otherwise it implements the appropriate
 *	buffered I/O behaviour with net_filbuf() and net_flsbuf()
 *	calling read_net() and write_net() respectively.
 *	
 * SEE ALSO:
 *	rwnet.c
 *	 
 * AUTHOR:
 *	Simon J. Gerraty <sjg@quick.com.au>
 *	
 *	This is a re-work of a stdio to socket implementation for DOS
 *	I did back in the late 1980s.
 */
/*
 *      @(#)Copyright (c) 1989,1996 Simon J. Gerraty.
 *      
 *      This is free software.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@quick.com.au
 */
#ifndef lint
static char RCSid[] = "$Id: netfgetln.c,v 1.3 1997/04/19 03:10:12 sjg Exp $";
#endif

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <sys/param.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>

#include "netiob.h"

#ifndef BSD4_4
extern char *fgetln __P((FILE *, size_t *));
#endif

/*
 * this is a crok, but should do.
 */
char *
net_fgetln(fp, n)
	FILE *fp;
	size_t *n;
{
#ifdef NEED_NETIOB
	NETIOB *p = net_fgetiob(fp);
	static char buf[2 * BUFSIZ];
	
	if (!p)
		return fgetln(fp, n);
	
	if (net_fgets(buf, sizeof(buf), fp)) {
		*n = strlen(buf);
		return buf;
	}
	return 0;
#else
	return fgetln(fp, n);
#endif
}


