/*
 * NAME:
 *	rwnet - recv/send with hooks
 *
 * DESCRIPTION:
 *	These routines provide a means of instantiating crypto or
 *	other hooks.  They are much the same routines I used in stelnet.
 *
 *
 * AUTHOR:
 *	Simon J. Gerraty <sjg@quick.com.au>
 */
/*
 *      @(#)Copyright (c) 1996 Simon J. Gerraty.
 *      
 *      This is free software.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@quick.com.au
 */
#ifndef lint
static char RCSid[] = "$Id: rcvnet.c,v 1.6 1997/05/15 15:50:50 sjg Exp $";
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#ifdef USE_SSL
# ifdef SYS_SELECT_H
#   include <sys/select.h>
# endif

#include "x509.h"
#include "ssl.h"

#include "sslfd.h"

static SSL *ssl_con;
#endif

/*
 * These are needed to ensure that SSL_read() called from read_net()
 * does not call SSL_read() via read()
 */
extern int _rmyself, _wmyself;

int
recv_net(fd, buf, nbytes, flags)
	int fd;
	unsigned char *buf;
	int nbytes;
	int flags;
{
	RWHOOK *hp;
	int n = 0;

	++_rmyself;
#ifdef USE_SSL
	if (_rmyself == 1 && (ssl_con = ssl_getcon(fd))) {
		n = SSL_read(ssl_con, buf, nbytes);
	} else
#endif
		n = recv(fd, buf, nbytes, flags);
	if (_rmyself == 1 && (hp = get_hook(fd, 0)) && hp->read) {
		if (hp->flag & HK_INDIRECT)
			n = hp->read(fd, &buf, n, flags);
		else
			n = hp->read(fd, buf, n, flags);
	}
	--_rmyself;
	return n;
}

int
send_net(fd, buf, nbytes, flags)
	int fd;
	unsigned char *buf;
	int nbytes;
	int flags;
{
	RWHOOK *hp;
	int n;
	
	++_wmyself;
	if (_wmyself == 1 && (hp = get_hook(fd, 0)) && hp->write) {
		if (hp->flag & HK_INDIRECT)
			nbytes = hp->write(fd, &buf, nbytes, flags);
		else
			nbytes = hp->write(fd, buf, nbytes, flags);
	}
#ifdef USE_SSL
	if (_wmyself == 1 && (ssl_con = ssl_getcon(fd))) {
		n = SSL_write(ssl_con, buf, nbytes);
	} else
#endif
		n = send(fd, buf, nbytes, flags);
	--_wmyself;
	return n;
}
