/*
 * NAME:
 *	rwnet - read/write network
 *
 * DESCRIPTION:
 *	These routines provide a means of instantiating crypto or
 *	other hooks.  On most systems, we use syscall() to get to the
 *	real system call.  Where that is unavailable, we do some
 *	horrible hacks on object files extracted from libc.a
 *
 * AUTHOR:
 *	Simon J. Gerraty <sjg@quick.com.au>
 */
/*
 *      @(#)Copyright (c) 1996 Simon J. Gerraty.
 *      
 *      This is free software.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@quick.com.au
 */
#ifndef lint
static char RCSid[] = "$Id: rwnet.c,v 1.19 1998/03/24 04:35:43 sjg Exp $";
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/param.h>
#ifdef USE_SSL
# ifdef SYS_SELECT_H
#   include <sys/select.h>
# endif

#include "x509.h"
#include "ssl.h"

#include "sslfd.h"

static SSL *ssl_con;
#endif


/*
 * These are needed to ensure that SSL_read() called from recv_net()
 * does not call SSL_read() via _read()
 */
int _rmyself = 0, _wmyself = 0;

#if !defined(NO_SYSCALL)			/* rare? */
#include <sys/syscall.h>

#define _CLOSE(f)	syscall(SYS_close, (f))
#define _READ(f,b,n)	syscall(SYS_read, (f), (b), (n))
#define _WRITE(f,b,n)	syscall(SYS_write, (f), (b), (n))
#else

#ifdef HAVE_IO_H
# include "io.h"
#endif
#ifdef HAVE_cLoSe
# define _CLOSE cLoSe
#else
# define _CLOSE close
#endif
#ifdef HAVE_rEaD
# define _READ rEaD
#else
# define _READ read
#endif
#ifdef HAVE_wRiTe
# define _WRITE wRiTe
#else
# define _WRITE write
#endif
#endif


int
_close(fd)
	int fd;
{
	static int myself = 0;
	int (*fp)();
	int n;

	++myself;
	if (myself == 1 && (fp = get_hook(fd, HK_CLOSE)))
		fp(fd);
#ifdef USE_SSL
	if (myself == 1 && (ssl_con = ssl_getcon(fd))) {
		n = ssl_close(fd);
	} else
#endif
		n = _CLOSE(fd);
	--myself;
	return n;
}

int
_read(fd, buf, nbytes)
	int fd;
	unsigned char *buf;
	int nbytes;
{
	RWHOOK *hp;
	int n = 0;

	++_rmyself;
#ifdef USE_SSL
	if (_rmyself == 1 && (ssl_con = ssl_getcon(fd))) {
		n = SSL_read(ssl_con, buf, nbytes);
	} else
#endif
		n = _READ(fd, buf, nbytes);
	if (_rmyself == 1 && (hp = get_hook(fd, 0)) && hp->read) {
		if (hp->flag & HK_INDIRECT)
			n = hp->read(fd, &buf, n, 0);
		else
			n = hp->read(fd, buf, n, 0);
	}
	--_rmyself;
	return n;
}

int
_write(fd, buf, nbytes)
	int fd;
	unsigned char *buf;
	int nbytes;
{
	RWHOOK *hp;
	int n;

	++_wmyself;
	if (_wmyself == 1 && (hp = get_hook(fd, 0)) && hp->write) {
		if (hp->flag & HK_INDIRECT)
			nbytes = hp->write(fd, &buf, nbytes, 0);
		else
			nbytes = hp->write(fd, buf, nbytes, 0);
	}
#ifdef USE_SSL
	if (_wmyself == 1 && (ssl_con = ssl_getcon(fd))) {
		n = SSL_write(ssl_con, buf, nbytes);
	} else
#endif
	n = _WRITE(fd, buf, nbytes);
	--_wmyself;
	return n;
}

/*
 * we now include rwstub.c here
 * to ensure that we link with our _read() etc not
 * those in libc.
 * If your OS has other things like _libc_read() define
 * them via rwxtra.h below
 */
#undef CLOSE
#undef READ
#undef WRITE
#define CLOSE close_net
#define READ read_net
#define WRITE write_net
#include "rwstub.c"
/*
 * pick up other stubs
 */
#include "rwxtra.h"

/*
 * if NEED_NETIOB is defined, we cannot do read/write
 */
#if !defined(SOCKS) && !(defined(READ) && defined(NEED_NETIOB))
#undef CLOSE
#undef READ
#undef WRITE
#define CLOSE close
#define READ read
#define WRITE write
#include "rwstub.c"
#endif /* READ && NEED_NETIOB */
